@@ -1,6 +1,6 @@
 /* Header file for GDB command decoding library.
 
-   Copyright (C) 2000-2017 Free Software Foundation, Inc.
+   Copyright (C) 2000-2019 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,16 +15,17 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#if !defined (CLI_DECODE_H)
-#define CLI_DECODE_H 1
+#ifndef CLI_CLI_DECODE_H
+#define CLI_CLI_DECODE_H
 
 /* This file defines the private interfaces for any code implementing
    command internals.  */
 
 /* Include the public interfaces.  */
 #include "command.h"
-
-struct re_pattern_buffer;
+#include "gdb_regex.h"
+#include "cli-script.h"
+#include "completer.h"
 
 #if 0
 /* FIXME: cagney/2002-03-17: Once cmd_type() has been removed, ``enum
@@ -46,8 +47,38 @@ cmd_types;
 
 struct cmd_list_element
   {
+    cmd_list_element (const char *name_, enum command_class theclass_,
+		      const char *doc_)
+      : name (name_),
+	theclass (theclass_),
+	cmd_deprecated (0),
+	deprecated_warn_user (0),
+	malloced_replacement (0),
+	doc_allocated (0),
+	name_allocated (0),
+	hook_in (0),
+	allow_unknown (0),
+	abbrev_flag (0),
+	type (not_set_cmd),
+	var_type (var_boolean),
+	doc (doc_)
+    {
+      memset (&function, 0, sizeof (function));
+    }
+
+    ~cmd_list_element ()
+    {
+      if (doc && doc_allocated)
+	xfree ((char *) doc);
+      if (name_allocated)
+	xfree ((char *) name);
+    }
+
+    DISABLE_COPY_AND_ASSIGN (cmd_list_element);
+
+
     /* Points to next command in this list.  */
-    struct cmd_list_element *next;
+    struct cmd_list_element *next = nullptr;
 
     /* Name of this command.  */
     const char *name;
@@ -81,6 +112,10 @@ struct cmd_list_element
 
     unsigned int doc_allocated : 1;
 
+    /* Set if the name field should be xfree'd.  */
+
+    unsigned int name_allocated : 1;
+
     /* Flag that specifies if this command is already running its hook.  */
     /* Prevents the possibility of hook recursion.  */
     unsigned int hook_in : 1;
@@ -108,21 +143,22 @@ struct cmd_list_element
        cagney/2002-02-02: This function signature is evolving.  For
        the moment suggest sticking with either set_cmd_cfunc() or
        set_cmd_sfunc().  */
-    void (*func) (struct cmd_list_element *c, char *args, int from_tty);
+    void (*func) (struct cmd_list_element *c, const char *args, int from_tty)
+      = nullptr;
     /* The command's real callback.  At present func() bounces through
        to one of the below.  */
     union
       {
 	/* If type is not_set_cmd, call it like this: */
-	cmd_cfunc_ftype *cfunc;
+	cmd_const_cfunc_ftype *const_cfunc;
 	/* If type is set_cmd or show_cmd, first set the variables,
 	   and then call this: */
-	cmd_sfunc_ftype *sfunc;
+	cmd_const_sfunc_ftype *sfunc;
       }
     function;
 
     /* Local state (context) for this command.  This can be anything.  */
-    void *context;
+    void *context = nullptr;
 
     /* Documentation of this command (or help topic).
        First line is brief documentation; remaining lines form, with it,
@@ -132,49 +168,37 @@ struct cmd_list_element
 
     /* For set/show commands.  A method for printing the output to the
        specified stream.  */
-    show_value_ftype *show_value_func;
+    show_value_ftype *show_value_func = nullptr;
 
     /* If this command is deprecated, this is the replacement name.  */
-    const char *replacement;
+    const char *replacement = nullptr;
 
     /* If this command represents a show command, then this function
        is called before the variable's value is examined.  */
-    void (*pre_show_hook) (struct cmd_list_element *c);
+    void (*pre_show_hook) (struct cmd_list_element *c) = nullptr;
 
     /* Hook for another command to be executed before this command.  */
-    struct cmd_list_element *hook_pre;
+    struct cmd_list_element *hook_pre = nullptr;
 
     /* Hook for another command to be executed after this command.  */
-    struct cmd_list_element *hook_post;
+    struct cmd_list_element *hook_post = nullptr;
 
     /* Nonzero identifies a prefix command.  For them, the address
        of the variable containing the list of subcommands.  */
-    struct cmd_list_element **prefixlist;
+    struct cmd_list_element **prefixlist = nullptr;
 
     /* For prefix commands only:
        String containing prefix commands to get here: this one
        plus any others needed to get to it.  Should end in a space.
        It is used before the word "command" in describing the
        commands reached through this prefix.  */
-    const char *prefixname;
+    const char *prefixname = nullptr;
 
     /* The prefix command of this command.  */
-    struct cmd_list_element *prefix;
+    struct cmd_list_element *prefix = nullptr;
 
-    /* Completion routine for this command.  TEXT is the text beyond
-       what was matched for the command itself (leading whitespace is
-       skipped).  It stops where we are supposed to stop completing
-       (rl_point) and is '\0' terminated.
-
-       Return value is a malloc'd vector of pointers to possible
-       completions terminated with NULL.  If there are no completions,
-       returning a pointer to a NULL would work but returning NULL
-       itself is also valid.  WORD points in the same buffer as TEXT,
-       and completions should be returned relative to this position.
-       For example, suppose TEXT is "foo" and we want to complete to
-       "foobar".  If WORD is "oo", return "oobar"; if WORD is
-       "baz/foo", return "baz/foobar".  */
-    completer_ftype *completer;
+    /* Completion routine for this command.  */
+    completer_ftype *completer = symbol_completer;
 
     /* Handle the word break characters for this completer.  Usually
        this function need not be defined, but for some types of
@@ -182,48 +206,47 @@ struct cmd_list_element
        a class) the word break chars may need to be redefined
        depending on the completer type (e.g., for filename
        completers).  */
-
-    completer_ftype_void *completer_handle_brkchars;
+    completer_handle_brkchars_ftype *completer_handle_brkchars = nullptr;
 
     /* Destruction routine for this command.  If non-NULL, this is
        called when this command instance is destroyed.  This may be
        used to finalize the CONTEXT field, if needed.  */
-    void (*destroyer) (struct cmd_list_element *self, void *context);
+    void (*destroyer) (struct cmd_list_element *self, void *context) = nullptr;
 
     /* Pointer to variable affected by "set" and "show".  Doesn't
        matter if type is not_set.  */
-    void *var;
+    void *var = nullptr;
 
     /* Pointer to NULL terminated list of enumerated values (like
        argv).  */
-    const char *const *enums;
+    const char *const *enums = nullptr;
 
     /* Pointer to command strings of user-defined commands */
-    struct command_line *user_commands;
+    counted_command_line user_commands;
 
     /* Pointer to command that is hooked by this one, (by hook_pre)
        so the hook can be removed when this one is deleted.  */
-    struct cmd_list_element *hookee_pre;
+    struct cmd_list_element *hookee_pre = nullptr;
 
     /* Pointer to command that is hooked by this one, (by hook_post)
        so the hook can be removed when this one is deleted.  */
-    struct cmd_list_element *hookee_post;
+    struct cmd_list_element *hookee_post = nullptr;
 
     /* Pointer to command that is aliased by this one, so the
        aliased command can be located in case it has been hooked.  */
-    struct cmd_list_element *cmd_pointer;
+    struct cmd_list_element *cmd_pointer = nullptr;
 
     /* Start of a linked list of all aliases of this command.  */
-    struct cmd_list_element *aliases;
+    struct cmd_list_element *aliases = nullptr;
 
     /* Link pointer for aliases on an alias list.  */
-    struct cmd_list_element *alias_chain;
+    struct cmd_list_element *alias_chain = nullptr;
 
     /* If non-null, the pointer to a field in 'struct
        cli_suppress_notification', which will be set to true in cmd_func
        when this command is being executed.  It will be set back to false
        when the command has been executed.  */
-    int *suppress_notification;
+    int *suppress_notification = nullptr;
   };
 
 extern void help_cmd_list (struct cmd_list_element *, enum command_class,
@@ -234,18 +257,27 @@ extern void help_cmd_list (struct cmd_list_element *, enum command_class,
 extern void help_cmd (const char *, struct ui_file *);
 
 extern void apropos_cmd (struct ui_file *, struct cmd_list_element *,
-                         struct re_pattern_buffer *, const char *);
+			 bool verbose, compiled_regex &, const char *);
 
 /* Used to mark commands that don't do anything.  If we just leave the
    function field NULL, the command is interpreted as a help topic, or
    as a class of commands.  */
 
-extern void not_just_help_class_command (char *arg, int from_tty);
+extern void not_just_help_class_command (const char *arg, int from_tty);
 
-/* Exported to cli/cli-setshow.c */
+/* Print only the first line of STR on STREAM.
+   FOR_VALUE_PREFIX true indicates that the first line is output
+   to be a prefix to show a value (see deprecated_show_value_hack):
+   the first character is printed in uppercase, and the trailing
+   dot character is not printed.  */
 
-extern void print_doc_line (struct ui_file *, const char *);
+extern void print_doc_line (struct ui_file *stream, const char *str,
+			    bool for_value_prefix);
 
+/* The enums of boolean commands.  */
+extern const char * const boolean_enums[];
+
+/* The enums of auto-boolean commands.  */
 extern const char * const auto_boolean_enums[];
 
 /* Verify whether a given cmd_list_element is a user-defined command.
@@ -255,4 +287,4 @@ extern int cli_user_command_p (struct cmd_list_element *);
 
 extern int find_command_name_length (const char *);
 
-#endif /* !defined (CLI_DECODE_H) */
+#endif /* CLI_CLI_DECODE_H */
